/**
 * Copyright (c) 1993 Richard V. Nash.
 * Copyright (c) 2000 Dan Papasian.  All rights reserved.
 * Copyright (c) 2002 Andrew Tridgell <tridge@samba.org>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


/*
  ficsmain.c is a minimal wrapper around chessd.so. By keeping this module to a
  minimum size we can reload nearly all the chess server functionality on a 'areload'
  command. 
*/


#define EXTERN
#include "includes.h"

/* handle used to talk to chessd.so */
static void *chessd_so_handle;

static void usage();
static void main_event_loop(void);

/* 
   load a function from chessd.so 
   this is the core of the runtime reload functionality
*/
static void *chessd_function(const char *name)
{
	void *sym;

	if (!chessd_so_handle) {
		chessd_so_handle = dlopen(LIB_DIR "/chessd.so", RTLD_LAZY);
		if (!chessd_so_handle) {
			fprintf(stderr, "CHESSD: Failed to load chessd.so !! (%s)\n",
				dlerror());
			exit(1);
		}
	}

	sym = dlsym(chessd_so_handle, name);
	if (!sym) {
		fprintf(stderr, "CHESSD: Failed to find symbol %s in chessd.so !!\n",
			name);
		exit(1);
	}

	return sym;
}


static void usage()
{
	fprintf(stderr,  "Usage: chessd -p port [-f] [-T timeseal_decoder] [-R rootdir]\n");
	fprintf(stderr,  "PORT_NUMBER is 5000 by default\n");
	exit(1);
}

static int daemonize(void)
{
	pid_t pid;
	if ((pid = fork()) == -1) 
		return -1;
	else if (pid != 0)
		exit(0);
	if (setsid() == -1)
		return -1;
	return 0;
}

static void main_event_loop(void) 
{
	void (*select_loop)(void ) = chessd_function("select_loop");

	while (1) {
		select_loop();

		/* check the reload flag */
		if (chessd_reload_flag) {
			void (*reload_close)(void ) = chessd_function("reload_close");
			void (*reload_open)(void );

			chessd_reload_flag = 0;

			fprintf(stderr, "CHESSD: Reloading server code!\n");

			/* free up local vars */
			reload_close();

			/* close the handle to the shared lib */
			dlclose(chessd_so_handle);
			chessd_so_handle = NULL;

			/* re-initialise local variables */
			reload_open = chessd_function("reload_open");
			reload_open();
			select_loop = chessd_function("select_loop");
		}
	}
}

static void TerminateServer(int sig)
{
	void (*output_shut_mess)(void ) = chessd_function("output_shut_mess");
	void (*TerminateCleanup)(void ) = chessd_function("TerminateCleanup");
	void (*net_close)(void ) = chessd_function("net_close");

	fprintf(stderr,  "CHESSD: Received signal %d\n", sig);
	output_shut_mess();
	TerminateCleanup();
	net_close();
	exit(1);
}

static void BrokenPipe(int sig)
{
	signal(SIGPIPE, BrokenPipe);
	fprintf(stderr,  "CHESSD: Pipe signal\n");
}


/* this assumes we are setuid root to start */
static void do_chroot(const char *dir)
{
	int i;
	uid_t uid = getuid();
	uid_t euid = geteuid();
	struct rlimit rlp;

	if (euid != 0 || setuid(0) != 0) {
		fprintf(stderr, "Must be setuid root to use -R\n");
		exit(1);
	}
	if (chroot(dir) != 0 || chdir("/") != 0) {
		perror("chroot");
		exit(1);
	}
	if (setuid(uid) != 0) {
		perror("setuid");
		exit(1);
	}

	/* close all extraneous file descriptors */
	for (i=0;i<3;i++) close(i);
	open("stdin.log", O_RDWR|O_CREAT|O_TRUNC, 0644);
	open("stdout.log", O_WRONLY|O_CREAT|O_TRUNC, 0644);
	open("stderr.log", O_WRONLY|O_CREAT|O_TRUNC, 0644);

	/* reestabish core limits */
	getrlimit(RLIMIT_CORE, &rlp );
	rlp.rlim_cur = MAX(64*1024*1024, rlp.rlim_max);
	setrlimit( RLIMIT_CORE, &rlp );
}

int main(int argc, char *argv[])
{
	int i, foreground, port;
	void (*timeseal_init)(const char * ) = chessd_function("timeseal_init");
	void (*player_array_init)(void ) = chessd_function("player_array_init");
	int (*net_init)(int ) = chessd_function("net_init");
	const char *(*strltime)(time_t *) = chessd_function("strltime");
	void (*reload_open)(void ) = chessd_function("reload_open");

	port = DEFAULT_PORT;
	foreground = 0;

	/* enable malloc checking in libc */
	setenv("MALLOC_CHECK_", "2", 1);
	
	while((i = getopt(argc, argv, "p:fR:T:")) != -1) {
		switch(i) {
		case 'p':
			port = atoi(optarg);
			break;
		case 'f':
			foreground = 1;
			break;
		case 'T':
			timeseal_init(optarg);
			break;
		case 'R':
			do_chroot(optarg);
			break;
		default:
			usage();
		}
	}
   
	if (!foreground && daemonize()){
		printf("Problem with Daemonization - Aborting\n");
		exit(1);
	}  

	signal(SIGTERM, TerminateServer);
	signal(SIGINT, TerminateServer);
	signal(SIGPIPE, BrokenPipe);

	if (net_init(port)) {
		fprintf(stderr,  "CHESSD: Network initialize failed on port %d.\n", port);
		exit(1);
	}
	command_globals.startuptime = time(0);

	seek_globals.quota_time = 60;
	command_globals.player_high = 0;
	command_globals.game_high = 0;
	srandom(command_globals.startuptime);
	fprintf(stderr,  "CHESSD: Initialized on port %d at %s.\n", 
		port, strltime(&command_globals.startuptime));

	fprintf(stderr,  "CHESSD: player_array_init()\n");
	player_array_init();

	reload_open();
	
	main_event_loop();
	/* will never get here - uses TerminateServer */
	
	return 0;
}
