/**
 * Copyright (c) 1993 Richard V. Nash.
 * Copyright (c) 2000 Dan Papasian.  All rights reserved.
 *
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include "includes.h"

static int safestring(char *str);
static FILE *fopen_dfile(const char *dir, char *file, const char *mode);

char *eatword(char *str)
{
  while (*str && !isspace(*str))
    str++;
  return str;
}

char *eatwhite(char *str)
{
  while (*str && isspace(*str))
    str++;
  return str;
}

char *eattailwhite(char *str)
{
  int len;
  if (str == NULL)
    return NULL;

  len = strlen(str);
  while (len > 0 && isspace(str[len - 1]))
    len--;
  str[len] = '\0';
  return (str);
}

char *nextword(char *str)
{
  return eatwhite(eatword(str));
}

int mail_string_to_address(char *addr, char *subj, char *str)
{
  int fd;
  char template[] = SPOOL_DIR;

  fd = mkstemp(template);
  if (fd == -1) {
    d_printf("Failed to create spool file\n");
    return -1;
  }

  dprintf(fd, "To: %s\nSubject: %s\n\n%s\n", addr, subj, str);
  close(fd);

  return 0;
}

int mail_string_to_user(int p, char *subj, char *str)
{

  if (player_globals.parray[p].emailAddress &&
      player_globals.parray[p].emailAddress[0] &&
      safestring(player_globals.parray[p].emailAddress)) {
    return mail_string_to_address(player_globals.parray[p].emailAddress, subj, str);
  } else {
    return -1;
  }
}


/* Process a command for a user */
int pcommand(int p, char *comstr,...)
{
  char *tmp;
  int retval;
  int current_socket = player_globals.parray[p].socket;
  va_list ap;
  va_start(ap, comstr);

  vasprintf(&tmp, comstr, ap);
  retval = process_input(current_socket, tmp);
  free(tmp);
  if (retval == COM_LOGOUT) {
    process_disconnection(current_socket);
    net_close_connection(current_socket);
  }
  va_end(ap);
  return retval;
}

static int vpprintf(int p, int do_formatting, char *format,va_list ap)
{
	char *tmp = NULL;
	int retval;

	retval = vasprintf(&tmp, format, ap);
	if (retval != 0) {
		net_send_string(player_globals.parray[p].socket, 
				tmp, 
				do_formatting, 
				player_globals.parray[p].d_width + 1);
	}
	if (tmp) {
		free(tmp);
	}

	return retval;
}

int pprintf(int p, char *format,...)
{
	int retval;
	va_list ap;
	va_start(ap, format);
	retval = vpprintf(p, 1, format, ap);
	va_end(ap);
	return retval;
}

static void pprintf_dohightlight(int p)
{
	if (player_globals.parray[p].highlight & 0x01)
		pprintf(p, "\033[7m");
	if (player_globals.parray[p].highlight & 0x02)
		pprintf(p, "\033[1m");
	if (player_globals.parray[p].highlight & 0x04)
		pprintf(p, "\033[4m");
	if (player_globals.parray[p].highlight & 0x08)
		pprintf(p, "\033[2m");
}

int pprintf_highlight(int p, char *format,...)
{
	int retval;
	va_list ap;

	pprintf_dohightlight(p);
	va_start(ap, format);
	retval = vpprintf(p, 1, format, ap);
	va_end(ap);
	if (player_globals.parray[p].highlight)
		pprintf(p, "\033[0m");
	return retval;
}

static void sprintf_dohightlight(int p, char *s)
{
	if (player_globals.parray[p].highlight & 0x01)
		strcat(s, "\033[7m");
	if (player_globals.parray[p].highlight & 0x02)
		strcat(s, "\033[1m");
	if (player_globals.parray[p].highlight & 0x04)
		strcat(s, "\033[4m");
	if (player_globals.parray[p].highlight & 0x08)
		strcat(s, "\033[2m");
}

int psprintf_highlight(int p, char *s, char *format,...)
{
	int retval;
	va_list ap;
	
	va_start(ap, format);
	if (player_globals.parray[p].highlight) {
		sprintf_dohightlight(p, s);
		retval = vsprintf(s + strlen(s), format, ap);
		strcat(s, "\033[0m");
	} else {
		retval = vsprintf(s, format, ap);
	}
	va_end(ap);
	return retval;
}

int pprintf_prompt(int p, char *format,...)
{
	int retval;
	va_list ap;
	va_start(ap, format);
	retval = vpprintf(p, 1, format, ap);
	va_end(ap);
	send_prompt(p);
	return retval;
}

/* send a prompt to p */
void send_prompt(int p) 
{
	char *prompt = player_globals.parray[p].prompt;
	pprintf(p, "%s%s", 
		prompt,
		isspace(prompt[strlen(prompt)-1])?"":" ");
}

int pprintf_noformat(int p, char *format,...)
{
	int retval;
	va_list ap;
	va_start(ap, format);
	retval = vpprintf(p, 0, format, ap);
	va_end(ap);
	return retval;
}

void Bell (int p)
{
  if (CheckPFlag(p, PFLAG_BELL))
    pprintf (p, "\a");
  return;
}

int psend_raw_file(int p, char *dir, char *file)
{
  FILE *fp;
  char tmp[MAX_LINE_SIZE];
  int num;

  fp = fopen_dfile(dir, file, "r");

  if (!fp)
    return -1;
  while ((num = fread(tmp, sizeof(char), MAX_LINE_SIZE - 1, fp)) > 0) {
    tmp[num] = '\0';
    net_send_string(player_globals.parray[p].socket, tmp, 1, player_globals.parray[p].d_width + 1);
  }
  fclose(fp);
  return 0;
}

int psend_file(int p, const char *dir, const char *file)
{
  FILE *fp;
  char tmp[MAX_LINE_SIZE];
  char *fname;
  int lcount = player_globals.parray[p].d_height - 1;

  if (player_globals.parray[p].last_file)
    free(player_globals.parray[p].last_file);
  player_globals.parray[p].last_file = NULL;
  player_globals.parray[p].last_file_byte = 0L;

 /* Can't use fopen_dfile because we need fname */
  if(dir)
    asprintf(&fname,"%s/%s",dir,file);
  else
    fname = strdup(file);

  fp = fopen(fname, "r");
  if (!fp)
    return -1;
  while (!feof(fp) && (--lcount > 0)) {
    fgets(tmp, MAX_LINE_SIZE - 1, fp);
    if (!feof(fp)) {
      net_send_string(player_globals.parray[p].socket, tmp, 1, player_globals.parray[p].d_width + 1);
    }
  }
  if (!feof(fp)) {
    player_globals.parray[p].last_file = strdup(fname);
    player_globals.parray[p].last_file_byte = ftell(fp);
    pprintf(p, "Type [next] to see next page.\n");
  }

  free(fname);
  fclose(fp);
  return 0;
}

/* 
 * Marsalis added on 8/27/95 so that [next] does not
 * appear in the logout process for those that have
 * a short screen height.  (Fixed due to complaint 
 * in Bug's messages).
 */
int psend_logoutfile(int p, char *dir, char *file)
{
  FILE *fp;
  char tmp[MAX_LINE_SIZE];
  
  if (player_globals.parray[p].last_file)
    free(player_globals.parray[p].last_file);
  player_globals.parray[p].last_file = NULL;
  player_globals.parray[p].last_file_byte = 0L;

  fp = fopen_dfile(dir, file, "r");
  if (!fp)
    return -1;
  
  while (!feof(fp)) {
    fgets(tmp, MAX_LINE_SIZE - 1, fp);
    if (!feof(fp)) {
	net_send_string(player_globals.parray[p].socket, tmp, 1,player_globals.parray[p].d_width + 1);
    }
  }

  fclose(fp);
  return 0;
}

int pmore_file(int p)
{
  FILE *fp;
  char tmp[MAX_LINE_SIZE];
  int lcount = player_globals.parray[p].d_height - 1;

  if (!player_globals.parray[p].last_file) {
    pprintf(p, "There is no more.\n");
    return -1;
  }
  fp = fopen(player_globals.parray[p].last_file, "r");
  if (!fp) {
    pprintf(p, "File not found!\n");
    return -1;
  }
  fseek(fp, player_globals.parray[p].last_file_byte, SEEK_SET);

  while (!feof(fp) && (--lcount > 0)) {
    fgets(tmp, MAX_LINE_SIZE, fp);
    if (!feof(fp)) {
	net_send_string(player_globals.parray[p].socket, tmp, 1, player_globals.parray[p].d_width + 1);
    }
  }
  if (!feof(fp)) {
    player_globals.parray[p].last_file_byte = ftell(fp);
    pprintf(p, "Type [next] to see next page.\n");
  } else {
    free(player_globals.parray[p].last_file);
    player_globals.parray[p].last_file = NULL;
    player_globals.parray[p].last_file_byte = 0L;
  }
  fclose(fp);
  return 0;
}

char *stolower(char *str)
{
  int i;

  if (!str)
    return NULL;
  for (i = 0; str[i]; i++) {
    if (isupper(str[i])) {
      str[i] = tolower(str[i]);
    }
  }
  return str;
}

static int safechar(int c)
{
  if ((c == '>') || (c == '!') || (c == '&') || (c == '*') || (c == '?') ||
      (c == '/') || (c == '<') || (c == '|') || (c == '`') || (c == '$'))
    return 0;
  return 1;
}

static int safestring(char *str)
{
  int i;

  if (!str)
    return 1;
  for (i = 0; str[i]; i++) {
    if (!safechar(str[i]))
      return 0;
  }
  return 1;
}

int alphastring(char *str)
{
  int i;

  if (!str)
    return 1;
  for (i = 0; str[i]; i++) {
    if (!isalpha(str[i])) {
      return 0;
    }
  }
  return 1;
}

int printablestring(char *str)
{
  int i;

  if (!str)
    return 1;
  for (i = 0; str[i]; i++) {
    if ((!isprint(str[i])) && (str[i] != '\t') && (str[i] != '\n'))
      return 0;
  }
  return 1;
}

char *hms_desc(int t)
{
static char tstr[80];
int days, hours, mins, secs;

    days  = (t / (60*60*24));
    hours = ((t % (60*60*24)) / (60*60));
    mins  = (((t % (60*60*24)) % (60*60)) / 60);
    secs  = (((t % (60*60*24)) % (60*60)) % 60);
    if ((days==0) && (hours==0) && (mins==0)) {
      sprintf(tstr, "%d sec%s",
                 secs, (secs==1) ? "" : "s");
    } else if ((days==0) && (hours==0)) {
/*      sprintf(tstr, "%d min%s, %d sec%s",
                 mins, (mins==1) ? "" : "s",
                 secs, (secs==1) ? "" : "s");  */
      sprintf(tstr, "%d min%s",
                 mins, (mins==1) ? "" : "s");
    } else if (days==0) {
      sprintf(tstr, "%d hr%s, %d min%s, %d "
                 "sec%s",
                 hours, (hours==1) ? "" : "s",
                 mins, (mins==1) ? "" : "s",
                 secs, (secs==1) ? "" : "s");
    } else {
      sprintf(tstr, "%d day%s, %d hour%s, %d minute%s "
                 "and %d second%s",
                 days, (days==1) ? "" : "s",
                 hours, (hours==1) ? "" : "s",
                 mins, (mins==1) ? "" : "s",
                 secs, (secs==1) ? "" : "s");
    }
    return tstr;
}

char *hms(int t, int showhour, int showseconds, int spaces)
{
  static char tstr[20];
  char tmp[10];
  int h, m, s;

  h = t / 3600;
  t = t % 3600;
  m = t / 60;
  s = t % 60;
  if (h || showhour) {
    if (spaces)
      sprintf(tstr, "%d : %02d", h, m);
    else
      sprintf(tstr, "%d:%02d", h, m);
  } else {
    sprintf(tstr, "%d", m);
  }
  if (showseconds) {
    if (spaces)
      sprintf(tmp, " : %02d", s);
    else
      sprintf(tmp, ":%02d", s);
    strcat(tstr, tmp);
  }
  return tstr;
}

/* This is used only for relative timeing since it reports seconds since
 * about 5:00pm on Feb 16, 1994
 */
unsigned tenth_secs(void)
{
  struct timeval tp;
  struct timezone tzp;

  gettimeofday(&tp, &tzp);
/* .1 seconds since 1970 almost fills a 32 bit int! So lets subtract off
 * the time right now */
  return ((tp.tv_sec - 331939277) * 10L) + (tp.tv_usec / 100000);
}

/* This is to translate tenths-secs time back into 1/1/70 time in full
 * seconds, because vek didn't read utils.c when he programmed new ratings.
   1 sec since 1970 fits into a 32 bit int OK.
*/
int untenths(unsigned tenths)
{
  return (tenths / 10 + 331939277 + 0xffffffff / 10 + 1);
}

char *tenth_str(unsigned t, int spaces)
{
  return hms((t + 5) / 10, 0, 1, spaces);	/* Round it */
}

#define MAX_TRUNC_SIZE 100

/* Warning, if lines in the file are greater than 1024 bytes in length, this
   won't work! */
/* nasty bug fixed by mann, 3-12-95 */
int truncate_file(char *file, int lines)
{
  FILE *fp;
  int bptr = 0, trunc = 0, i;
  char tBuf[MAX_TRUNC_SIZE][MAX_LINE_SIZE];

  if (lines > MAX_TRUNC_SIZE)
    lines = MAX_TRUNC_SIZE;
  fp = fopen(file, "r");
  if (!fp)
    return 1;
  while (!feof(fp)) {
    fgets(tBuf[bptr], MAX_LINE_SIZE, fp);
    if (feof(fp))
      break;
    if (tBuf[bptr][strlen(tBuf[bptr]) - 1] != '\n') {	/* Line too long */
      fclose(fp);
      return -1;
    }
    bptr++;
    if (bptr == lines) {
      trunc = 1;
      bptr = 0;
    }
  }
  fclose(fp);
  if (trunc) {
    fp = fopen(file, "w");
    for (i = 0; i < lines; i++) {
      fputs(tBuf[bptr], fp);
      bptr++;
      if (bptr == lines) {
	bptr = 0;
      }
    }
    fclose(fp);
  }
  return 0;
}

/* Warning, if lines in the file are greater than 1024 bytes in length, this
   won't work! */
int lines_file(char *file)
{
  FILE *fp;
  int lcount = 0;
  char tmp[MAX_LINE_SIZE];

  fp = fopen(file, "r");
  if (!fp)
    return 0;
  while (!feof(fp)) {
    if (fgets(tmp, MAX_LINE_SIZE, fp))
      lcount++;
  }
  fclose(fp);
  return lcount;
}

int file_has_pname(char *fname, char *plogin)
{
  if (!strcmp(file_wplayer(fname), plogin))
    return 1;
  if (!strcmp(file_bplayer(fname), plogin))
    return 1;
  return 0;
}

char *file_wplayer(char *fname)
{
  static char tmp[MAX_FILENAME_SIZE];
  char *ptr;
  strcpy(tmp, fname);
  ptr = rindex(tmp, '-');
  if (!ptr)
    return "";
  *ptr = '\0';
  return tmp;
}

char *file_bplayer(char *fname)
{
  char *ptr;

  ptr = rindex(fname, '-');
  if (!ptr)
    return "";
  return ptr + 1;
}


/* Hey, leave this code alone.  "a" is always in network byte order,
   which is big-endian, even on a little-endian machine.  I fixed this
   code to handle that correctly a while ago, and someone
   gratuitiously changed it so that it would work correctly only on a
   big-endian machine (like a Sun).  I have now changed it back. --mann
*/
char *dotQuad(unsigned int a)
{
  static char tmp[20];
  unsigned char *aa = (unsigned char *) &a;

  sprintf(tmp, "%d.%d.%d.%d", aa[0], aa[1], aa[2], aa[3]);
  return tmp;
}

int available_space(void)
{
#if defined(SYSTEM_NEXT)
  struct statfs buf;

  statfs(PLAYER_DIR, &buf);
  return ((buf.f_bsize * buf.f_bavail));
#elif defined(SYSTEM_ULTRIX)
  struct fs_data buf;

  statfs(PLAYER_DIR, &buf);
  return ((1024 * buf.bfreen));
#elif defined(SYSTEM_SUN4)
/*
  struct statfs buf;

  statfs(PLAYER_DIR, buf);
  return (1024 * buf.f_bfree); */
  return 100000000;		/* Infinite space */
#else
   return 100000000;		/* Infinite space */
#endif
}

int file_exists(char *fname)
{
  FILE *fp;

  fp = fopen(fname, "r");
  if (!fp)
    return 0;
  fclose(fp);
  return 1;
}

char *ratstr(int rat)
{
  static int on = 0;
  static char tmp[20][10];

  if (on == 20)
    on = 0;
  if (rat) {
    sprintf(tmp[on], "%4d", rat);
  } else {
    sprintf(tmp[on], "----");

  }
  on++;
  return tmp[on - 1];
}

char *ratstrii(int rat, int p)
{
  static int on = 0;
  static char tmp[20][10];

  if (on == 20)
    on = 0;
  if (rat) {
    sprintf(tmp[on], "%4d", rat);
  } else if (CheckPFlag(p, PFLAG_REG)) {
    sprintf(tmp[on], "----");
  } else {
    sprintf(tmp[on], "++++");
  }
  on++;
  return tmp[on - 1];
}

struct t_tree {
  struct t_tree *left, *right;
  char name;			/* Not just 1 char - space for whole name */
};				/* is allocated.  Maybe a little cheesy? */

struct t_dirs {
  struct t_dirs *left, *right;
  time_t mtime;			/* dir's modification time */
  struct t_tree *files;
  char name;			/* ditto */
};

static char **t_buffer = NULL; /* pointer to next space in return buffer */
static int t_buffersize = 0;	/* size of return buffer */

/* fill t_buffer with anything matching "want*" in file tree t_tree */
static void t_sft(char *want, struct t_tree *t)
{
  if (t) {
    int cmp = strncmp(want, &t->name, strlen(want));
    if (cmp <= 0)		/* if want <= this one, look left */
      t_sft(want, t->left);
    if (t_buffersize && (cmp == 0)) {	/* if a match, add it to buffer */
      t_buffersize--;
      *t_buffer++ = &(t->name);
    }
    if (cmp >= 0)		/* if want >= this one, look right */
      t_sft(want, t->right);
  }
}

/* delete file tree t_tree */
static void t_cft(struct t_tree **t)
{
  if (*t) {
    t_cft(&(*t)->left);
    t_cft(&(*t)->right);
    free(*t);
    *t = NULL;
  }
}

/* make file tree for dir d */
static void t_mft(struct t_dirs *d)
{
  DIR *dirp;
  struct dirent *dp;
  struct t_tree **t;

  if ((dirp = opendir(&(d->name))) == NULL) {
    d_printf( "CHESSD:mft() couldn't opendir.\n");
  } else {
    while ((dp = readdir(dirp))) {
      t = &d->files;
      if (dp->d_name[0] != '.') {	/* skip anything starting with . */
	while (*t) {
	  if (strcmp(dp->d_name, &(*t)->name) < 0) {
	    t = &(*t)->left;
	  } else {
	    t = &(*t)->right;
	  }
	}
	*t = malloc(sizeof(struct t_tree) + strlen(dp->d_name));
	(*t)->right = (*t)->left = NULL;
	strcpy(&(*t)->name, dp->d_name);
      }
    }
    closedir(dirp);
  }
}

int search_directory(const char *dir, char *filter, char **buffer, int buffersize)
{
/* dir = directory to search
   filter = what to search for
   buffer = where to store pointers to matches
   buffersize = how many pointers will fit inside buffer */

  static struct t_dirs *ramdirs = NULL;
  struct t_dirs **i;
  int cmp;
  static char nullify = '\0';
  struct stat statbuf;

  t_buffer = buffer;
  t_buffersize = buffersize;

  if (!stat(dir, &statbuf)) {
    if (filter == NULL)		/* NULL becomes pointer to null string */
      filter = &nullify;
    i = &ramdirs;
    while (*i) {			/* find dir in dir tree */
      cmp = strcmp(dir, &(*i)->name);
      if (cmp == 0)
	break;
      else if (cmp < 0)
	i = &(*i)->left;
      else
	i = &(*i)->right;
    }
    if (!*i) {				/* if dir isn't in dir tree, add him */
      *i = malloc(sizeof(struct t_dirs) + strlen(dir));
      (*i)->left = (*i)->right = NULL;
      (*i)->files = NULL;
      strcpy(&(*i)->name, dir);
    }
    if ((*i)->files) {			/* delete any obsolete file tree */
      if ((*i)->mtime != statbuf.st_mtime) {
	t_cft(&(*i)->files);
      }
    }
    if ((*i)->files == NULL) {		/* if no file tree for him, make one */
      (*i)->mtime = statbuf.st_mtime;
      t_mft(*i);
    }
    t_sft(filter, (*i)->files);		/* finally, search for matches */
  }
  return (buffersize - t_buffersize);
}

int display_directory(int p, char **buffer, int count)
/* buffer contains 'count' string pointers */
{
  int i;
  multicol *m = multicol_start(count);

  for (i = 0; (i < count); i++)
    multicol_store(m, *buffer++);
  multicol_pprint(m, p, player_globals.parray[p].d_width, 1);
  multicol_end(m);
  return (i);
}

void CenterText (char *target, const char *text, int width, int pad)
{
  int left, len;
  char *format;

  len = strlen(text);
  left = (width + 1 - len) / 2;    /* leading blank space. */

  if (pad)
    asprintf (&format, "%%%ds%%-%ds", left, width-left);  /* e.g. "%5s%-10s" */
  else
    asprintf (&format, "%%%ds%%s", left);    /* e.g. "%5s%s" */
  sprintf (target, format, "", text);

  free(format);

  return;
}

/* Open a file given it's possible directory and filename, as well */
/* As mode */

static FILE *fopen_dfile(const char *dir, char *file, const char *mode)
{
  char *fname;
  FILE *fd;

  if(dir)
    asprintf(&fname, "%s/%s", dir, file);
  else
    fname = file;

  fd = fopen(fname, mode);
  free(fname);
  return fd;
}


/* block a signal */
void block_signal(int signum)
{
	sigset_t set;
	sigemptyset(&set);
	sigaddset(&set,signum);
	sigprocmask(SIG_BLOCK,&set,NULL);
}

/* unblock a signal */
void unblock_signal(int signum)
{
	sigset_t set;
	sigemptyset(&set);
	sigaddset(&set,signum);
	sigprocmask(SIG_UNBLOCK,&set,NULL);
}


int file_copy(const char *src, const char *dest)
{
  int fd1, fd2, n;
  char buf[1024];

  fd1 = open(src, O_RDONLY);
  if (fd1 == -1) return -1;

  unlink(dest);
  fd2 = open(dest, O_WRONLY|O_CREAT|O_EXCL, 0644);
  if (fd2 == -1) {
    close(fd1);
    return -1;
  }

  while ((n = read(fd1, buf, sizeof(buf))) > 0) {
    if (write(fd2, buf, n) != n) {
      close(fd2);
      close(fd1);
      unlink(dest);
      return -1;
    }
  }

  close(fd1);
  close(fd2);
  return 0;
}


#ifndef HAVE_DPRINTF
 int dprintf(int fd, const char *format, ...)
{
	va_list ap;
	char *ptr = NULL;
	int ret = 0;

	va_start(ap, format);
	vasprintf(&ptr, format, ap);
	va_end(ap);

	if (ptr) {
		ret = write(fd, ptr, strlen(ptr));
		free(ptr);
	}

	return ret;
}
#endif


#ifndef HAVE_STRNLEN
/*
  some platforms don't have strnlen
*/
 size_t strnlen(const char *s, size_t n)
{
	int i;
	for (i=0; s[i] && i<n; i++) /* noop */ ;
	return i;
}
#endif


static const char *strtime(struct tm * stm, short gmt)
{
	static char tstr[100];

	if (gmt)
		strftime(tstr, sizeof(tstr), "%a %b %e, %H:%M GMT %Y", stm);
	else
		strftime(tstr, sizeof(tstr), "%a %b %e, %H:%M %Z %Y", stm);
	return (tstr);
}

const char *strltime(time_t *clock)
{
	struct tm *stm = localtime(clock);
	return strtime(stm, 0);
}

const char *strgtime(time_t *clock)
{
	struct tm *stm = gmtime(clock);
	return strtime(stm, 1);
}

/* useful debug utility */
void d_printf(const char *fmt, ...)
{
  va_list ap;
  time_t t = time(NULL);
  fprintf(stderr,"%s ", strltime(&t));
  va_start(ap, fmt);
  vfprintf(stderr,fmt,ap);
  va_end(ap);
}
